from typing import Callable, Dict

from jinja2 import Environment

from ..common import (
    camel_case,
    escape_backticks,
    escape_double_quotes,
    escape_single_quotes,
    full_to_half,
    make_append_func_to_dict_deco,
)

all_filters: Dict[str, Callable] = {}

__append_filter = make_append_func_to_dict_deco(all_filters)


__append_filter(camel_case)
__append_filter(full_to_half)
__append_filter(escape_backticks)
__append_filter(escape_double_quotes)
__append_filter(escape_single_quotes)


@__append_filter
def br(value: str) -> str:
    return value.replace("\n", "<br/>")


def register_all_filters(env: Environment):
    env.filters.update(all_filters)



import time
from typing import TYPE_CHECKING, Any
from typing_extensions import override

from nonebot.adapters.satori.models import LoginStatus

if TYPE_CHECKING:
    from nonebot.adapters.satori import Adapter as SatoriAdapter
    from nonebug import App
    from nonebug.mixin.process import MatcherContext


def create_fake_satori_adapter(ctx: "MatcherContext") -> "SatoriAdapter":
    from nonebot import get_driver
    from nonebot.adapters import Bot as BaseBot
    from nonebot.adapters.satori import Adapter as SatoriAdapter

    class FakeAdapter(SatoriAdapter):
        @override
        async def _call_api(self, bot: BaseBot, api: str, **data) -> Any:
            return ctx.got_call_api(self, api, **data)

    return FakeAdapter(get_driver())


async def test_captured_recall(app: "App"):
    from nonebot import on_message, require
    from nonebot.adapters.satori import Bot as SatoriBot
    from nonebot.adapters.satori.config import ClientInfo
    from nonebot.adapters.satori.event import MessageCreatedEvent
    from nonebot.adapters.satori.models import ChannelType, Login, MessageObject, User
    from nonebot.compat import type_validate_python

    require("nonebot_plugin_alconna")

    from cookit.nonebot.alconna import RecallContext
    from nonebot_plugin_alconna.uniseg import UniMessage

    matcher = on_message()

    @matcher.handle()
    async def _():
        async with RecallContext(delay=0, wait=True) as recall:
            await recall.send(UniMessage("world!"))
            recall.append(await UniMessage("world!").send())

    async with app.test_matcher(matcher) as ctx:
        adapter = create_fake_satori_adapter(ctx)
        bot = ctx.create_bot(
            base=SatoriBot,
            adapter=adapter,
            self_id="fake_bot",
            login=Login(
                user=User(id="fake_user"),
                platform="fake",
                self_id="fake_bot",
                status=LoginStatus.ONLINE,
            ),
            info=ClientInfo(port=14514),
        )
        event = type_validate_python(
            MessageCreatedEvent,
            {
                "id": 114514,
                "type": "message-created",
                "platform": "fake",
                "self_id": "fake_bot",
                "timestamp": int(time.time() * 1000),
                "channel": {"id": "fake_channel", "type": ChannelType.TEXT},
                "user": {"id": "fake_user"},
                "message": {"id": "fake_message", "content": "hello"},
            },
        )
        ctx.receive_event(bot, event)

        ctx.should_call_api(
            "message_create",
            {"channel_id": "fake_channel", "content": "world!"},
            [MessageObject(id="114515", content="world!")],
        )
        ctx.should_call_api(
            "message_create",
            {"channel_id": "fake_channel", "content": "world!"},
            [MessageObject(id="114516", content="world!")],
        )
        ctx.should_call_api(
            "message_delete",
            {"channel_id": "fake_channel", "message_id": "114515"},
        )
        ctx.should_call_api(
            "message_delete",
            {"channel_id": "fake_channel", "message_id": "114516"},
        )



import asyncio
from functools import wraps
from typing import (
    AsyncIterable,
    Callable,
    Generic,
    List,
    Optional,
    Protocol,
    TypedDict,
    TypeVar,
)
from typing_extensions import NotRequired, Unpack

T = TypeVar("T")


class PaginationCallable(Protocol, Generic[T]):
    async def __call__(self, page_size: int, offset: int) -> Optional[List[T]]: ...


class IterPFKwargs(TypedDict):
    """分页查询相关参数"""

    page_size: NotRequired[int]
    """单次查询的数量，默认为 `100`"""
    offset: NotRequired[int]
    """查询的初始偏移量，默认为 `0`"""
    delay: NotRequired[float]
    """每次查询后的延迟时间，单位秒，默认为 `0`"""
    max_size: NotRequired[int]
    """最大结果数，返回的结果数永远不会超过这个值，默认为 `0`，即不限制"""


def iter_pagination_func(**kwargs: Unpack[IterPFKwargs]):
    page_size = kwargs.get("page_size", 100)
    offset = kwargs.get("offset", 0)
    delay = kwargs.get("delay", 0.0)
    max_size = kwargs.get("max_size", 0)
    has_max_size = max_size > 0

    def decorator(func: PaginationCallable[T]) -> Callable[[], AsyncIterable[T]]:
        @wraps(func)
        async def wrapper():
            now_offset = offset
            while True:
                # 如果声明了最大结果数，
                # 那么确保 本次查询的数量 不会超过 剩余的最大结果数
                now_page_size = (
                    min(page_size, max_size - now_offset) if has_max_size else page_size
                )

                resp = await func(now_page_size, now_offset)
                if not resp:
                    break  # 无结果结束迭代

                for x in resp:
                    yield x

                now_offset += page_size
                if max_size > 0 and now_offset >= max_size:
                    break

                if delay:
                    await asyncio.sleep(delay)

        return wrapper

    return decorator



import base64
from contextlib import suppress
from functools import partial
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    Iterator,
    List,
    Literal,
    Optional,
    Sequence,
    TypeVar,
    Union,
    cast,
    overload,
)
from typing_extensions import ParamSpec, TypeGuard

import fleep

T = TypeVar("T")

TA = TypeVar("TA")
TB = TypeVar("TB")
N = TypeVar("N")

K = TypeVar("K")
V = TypeVar("V")

P = ParamSpec("P")
R = TypeVar("R")

LazyGetterType = Union[T, Callable[P, T]]


def lazy_get(
    val: "LazyGetterType[T, P]",
    *args: P.args,
    **kwargs: P.kwargs,
) -> T:
    return val(*args, **kwargs) if callable(val) else val


def qor(
    a: Union[TA, N],
    b: "LazyGetterType[TB, []]",
    none_val: N = None,
) -> Union[TA, TB]:
    def guard(x: Union[TA, N]) -> TypeGuard[TA]:
        return x is not none_val

    return a if guard(a) else lazy_get(b)


def chunks(lst: Sequence[T], n: int) -> Iterator[Sequence[T]]:
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def flatten(li: Iterable[Iterable[T]]) -> List[T]:
    return [x for y in li for x in y]


def set_default(target: Dict[K, V], key: K, default: "LazyGetterType[V, []]") -> V:
    if key in target:
        return target[key]
    default = lazy_get(default)
    target[key] = default
    return default


@overload
def auto_delete(target: Dict[K, V], transform: Literal[None] = None) -> Dict[K, V]: ...
@overload
def auto_delete(
    target: Dict[K, V],
    transform: Callable[[V], Optional[T]],
) -> Dict[K, T]: ...
def auto_delete(  # noqa: E302
    target: Dict[K, V],
    transform: Optional[Callable[[V], Optional[T]]] = None,
) -> Dict[K, Any]:
    data: Dict[K, Union[V, T]] = {}
    for k, v in tuple(target.items()):
        vt = transform(v) if transform else v
        if vt:
            data[k] = vt
        else:  # expected behavior, bool(vt) == False will be removed
            del target[k]
    return data


def to_b64_url(data: bytes, mime: Optional[str] = None) -> str:
    if mime is None:
        with suppress(IndexError):
            mime = cast(List[str], fleep.get(data[:128]).mime)[0]
    return f"data:{mime};base64,{base64.b64encode(data).decode()}"


@overload
def append_func_to_dict_deco(
    name_dict: Dict[str, Callable],
    func_or_name: Callable[P, R],
) -> Callable[P, R]: ...
@overload
def append_func_to_dict_deco(
    name_dict: Dict[str, Callable],
    func_or_name: str,
) -> Callable[[Callable[P, R]], Callable[P, R]]: ...
def append_func_to_dict_deco(
    name_dict: Dict[str, Callable],
    func_or_name: Union[Callable[P, R], str],
):
    if callable(func_or_name):
        name_dict[func_or_name.__name__] = func_or_name
        return func_or_name

    def inner_deco(func: Callable[P, R], /):
        name_dict[func_or_name] = func
        return func

    return inner_deco


def make_append_func_to_dict_deco(name_dict: Dict[str, Callable]):
    return partial(append_func_to_dict_deco, name_dict)



"""Install `cookit[nonebot-alconna]` before import this module."""

# region assert required plugin loaded
from .. import assert_plugin_loaded

assert_plugin_loaded("nonebot_plugin_alconna")
del assert_plugin_loaded
# endregion

# ruff: noqa: E402
from .receipt import (
    RecallContext as RecallContext,
)



import pytest


def pytest_configure(config: pytest.Config) -> None:
    from nonebug import NONEBOT_INIT_KWARGS

    config.stash[NONEBOT_INIT_KWARGS] = {
        "driver": "~fastapi+~websockets+~httpx",
        "log_level": "DEBUG",
    }


@pytest.fixture(scope="session", autouse=True)
def load_adapters(nonebug_init: None):  # noqa: ARG001, PT004
    from nonebot import get_driver
    from nonebot.adapters.satori import Adapter as SatoriAdapter

    driver = get_driver()
    driver.register_adapter(SatoriAdapter)



def test_camel_case_lower_first():
    from cookit import camel_case

    assert camel_case("hello_world") == "helloWorld"


def test_camel_case_upper_first():
    from cookit import camel_case

    assert camel_case("hello_world", upper_first=True) == "HelloWorld"


def test_camel_case_single_word():
    from cookit import camel_case

    assert camel_case("hello") == "hello"


def test_camel_case_empty_string():
    from cookit import camel_case

    assert camel_case("") == ""


def test_full_to_half():
    from cookit import full_to_half

    assert full_to_half("Ｈｅｌｌｏ　Ｗｏｒｌｄ！") == "Hello World!"
    assert full_to_half("１２３４５６７８９０") == "1234567890"
    assert full_to_half("ＡＢＣＤＥＦＧ") == "ABCDEFG"
    assert full_to_half("！＠＃＄％＾＆＊") == "!@#$%^&*"
    assert full_to_half("［］｛｝（）＜＞＝＋－／＊＆％＃＠") == "[]{}()<>=+-/*&%#@"
    assert (
        full_to_half("ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ")
        == "abcdefghijklmnopqrstuvwxyz"
    )
    assert (
        full_to_half("ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ")
        == "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    )
    assert (
        full_to_half(
            "１２３４５６７８９０！＠＃＄％＾＆＊（）＿＋｛｝［］：＂；＇＜＞，．／？",
        )
        == "1234567890!@#$%^&*()_+{}[]:\";'<>,./?"
    )


def test_escape_single_quotes():
    from cookit import escape_single_quotes

    assert escape_single_quotes("Hello, 'World'!") == "Hello, \\'World\\'!"

    assert escape_single_quotes("Hello, World!") == "Hello, World!"
    assert escape_single_quotes('Hello, "World"!') == 'Hello, "World"!'
    assert escape_single_quotes("Hello, `World`!") == "Hello, `World`!"


def test_escape_double_quotes():
    from cookit import escape_double_quotes

    assert escape_double_quotes('Hello, "World"!') == 'Hello, \\"World\\"!'

    assert escape_double_quotes("Hello, World!") == "Hello, World!"
    assert escape_double_quotes("Hello, 'World'!") == "Hello, 'World'!"
    assert escape_double_quotes("Hello, `World`!") == "Hello, `World`!"


def test_escape_backticks():
    from cookit import escape_backticks

    assert escape_backticks("Hello, `World`!") == "Hello, \\`World\\`!"

    assert escape_backticks("Hello, World!") == "Hello, World!"
    assert escape_backticks("Hello, 'World'!") == "Hello, 'World'!"
    assert escape_backticks('Hello, "World"!') == 'Hello, "World"!'



# generated by copilot

from cookit.pyd import CamelAliasModel, model_dump, type_validate_python


async def test_camel_alias_model():
    assert isinstance(CamelAliasModel(), CamelAliasModel)

    class TestModel(CamelAliasModel):
        test_field1: int
        test2: str

    data = {"testField1": 1, "test2": "2"}
    model = type_validate_python(TestModel, data)
    assert model_dump(model, by_alias=True) == data



"""Install `cookit[pydantic]` before import this module."""

from .compat import (
    PYDANTIC_V2 as PYDANTIC_V2,
    field_validator as field_validator,
    get_model_with_config as get_model_with_config,
    model_config as model_config,
    model_dump as model_dump,
    model_validator as model_validator,
    type_validate_json as type_validate_json,
    type_validate_python as type_validate_python,
)
from .util import (
    AliasFuncType as AliasFuncType,
    CamelAliasModel as CamelAliasModel,
    get_alias_model as get_alias_model,
)



from typing import TYPE_CHECKING, Callable, List, Optional, Type, Union

from loguru import logger

if TYPE_CHECKING:
    from loguru import Message, Record

    class NotRequiredRecord(Record, total=False):
        pass


LoguruHandler = Callable[["Message"], None]


def make_handler(log_stack: List["Message"]) -> LoguruHandler:
    def handler(message: "Message") -> None:
        log_stack.append(message)

    return handler


class LoggingContext:
    def __init__(self) -> None:
        self.log_stack: List["Message"] = []
        self.checker_stack: List[Callable[["Message"], None]] = []
        self.id: Optional[int] = None

    def __enter__(self):
        self.id = logger.add(make_handler(self.log_stack))
        return self

    def __exit__(self, *_):
        if self.id is not None:
            logger.remove(self.id)
        self.check()

    def should_log(
        self,
        *,
        exception: Union[BaseException, Type[BaseException], None] = ...,
        level_str: Optional[str] = None,
        level_no: Optional[int] = None,
        message: Optional[str] = None,
        message_fullmatch: bool = True,
    ):
        def checker(m: "Message"):
            record = m.record
            if exception is not ...:
                r_exception = record.get("exception")
                if exception is None:
                    assert r_exception is None
                else:
                    assert r_exception
                    assert (
                        r_exception.type is exception
                        if isinstance(exception, type)
                        else r_exception.value is exception
                    )
            if level_str is not None:
                assert record["level"].name == level_str
            if level_no is not None:
                assert record["level"].no == level_no
            if message is not None:
                r_message = record["message"]
                assert (
                    message == r_message if message_fullmatch else message in r_message
                )

        self.checker_stack.append(checker)

    def check(self):
        for checker in self.checker_stack:
            checker(self.log_stack.pop(0))



from typing import Any

import jinja2 as jj


def test_make_register_jinja_filter_deco():
    from cookit.jinja import make_register_jinja_filter_deco

    env = jj.Environment(autoescape=True)
    register_filter = make_register_jinja_filter_deco(env)

    def test_filter(val: Any) -> str:
        return f"test, {val}"

    register_filter(test_filter)
    register_filter("test_test_filter")(test_filter)

    assert env.filters["test_filter"] is test_filter
    assert env.filters["test_test_filter"] is test_filter



import asyncio
from asyncio import Semaphore
from typing import Any, Awaitable, Callable, Coroutine, TypeVar
from typing_extensions import ParamSpec

T = TypeVar("T")
R = TypeVar("R")
P = ParamSpec("P")


def with_semaphore(semaphore: Semaphore):
    def decorator(func: Callable[P, Awaitable[R]]):
        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
            async with semaphore:
                return await func(*args, **kwargs)

        return wrapper

    return decorator


def queued(func: Callable[P, Awaitable[R]]):
    return with_semaphore(Semaphore(1))(func)


async def race(*coros: Coroutine[Any, Any, T]) -> T:
    done, pending = await asyncio.wait(
        [asyncio.create_task(x) for x in coros],
        return_when=asyncio.FIRST_COMPLETED,
    )
    first, *other = done
    for t in (*other, *pending):
        t.cancel()
    return first.result()



from contextlib import contextmanager
from functools import partial
from typing import Iterator, Type, Union

from loguru import logger

from ..common import LazyGetterType, lazy_get


@contextmanager
def logged_suppress(
    msg: "LazyGetterType[str, [Exception]]",
    *t: Type[Exception],
    level: Union[int, str] = "ERROR",
    log_stack: bool = True,
    debug_stack: bool = False,
) -> Iterator[None]:
    try:
        yield
    except Exception as e:
        if t and (not isinstance(e, t)):
            raise
        lg = logger.opt(exception=e) if log_stack else logger
        lg.log(level, lazy_get(msg, e))
        if debug_stack:
            logger.opt(exception=e).debug("Stacktrace")
        return


warning_suppress = partial(
    logged_suppress,
    level="WARNING",
    log_stack=False,
    debug_stack=True,
)



import jinja2 as jj


def test_br():
    from cookit.jinja.filters import br

    assert br("hello\nworld") == "hello<br/>world"
    assert br("hello world") == "hello world"


def test_register_all_filters():
    from cookit.jinja import all_filters, register_all_filters

    env = jj.Environment(autoescape=True)
    register_all_filters(env)
    for k, v in all_filters.items():
        assert env.filters[k] is v



from .data import *  # noqa: F403
from .deprecated import *  # noqa: F403
from .math import (
    BYTE_B_UNITS as BYTE_B_UNITS,
    BYTE_UNITS as BYTE_UNITS,
    TIMEDELTA_EMPTY_DIVIDER as TIMEDELTA_EMPTY_DIVIDER,
    TIMEDELTA_NO_PAD as TIMEDELTA_NO_PAD,
    TIMEDELTA_SPACE_DIVIDER as TIMEDELTA_SPACE_DIVIDER,
    TIMEDELTA_SUFFIX_EN as TIMEDELTA_SUFFIX_EN,
    TIMEDELTA_SUFFIX_ZH as TIMEDELTA_SUFFIX_ZH,
    TimeUnit as TimeUnit,
    auto_convert_byte as auto_convert_byte,
    auto_convert_byte_b as auto_convert_byte_b,
    auto_convert_byte_b_speed as auto_convert_byte_b_speed,
    auto_convert_byte_speed as auto_convert_byte_speed,
    auto_convert_unit as auto_convert_unit,
    format_timedelta as format_timedelta,
    format_timedelta_human_en as format_timedelta_human_en,
    format_timedelta_human_en_spc as format_timedelta_human_en_spc,
    format_timedelta_human_zh as format_timedelta_human_zh,
    format_timedelta_human_zh_spc as format_timedelta_human_zh_spc,
)
from .other import (
    queued as queued,
    race as race,
    with_semaphore as with_semaphore,
)
from .text import (
    camel_case as camel_case,
    escape_backticks as escape_backticks,
    escape_double_quotes as escape_double_quotes,
    escape_single_quotes as escape_single_quotes,
    full_to_half as full_to_half,
)



from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    Literal,
    Optional,
    Set,
    Type,
    TypeVar,
    Union,
    overload,
)

from pydantic import VERSION, BaseModel, ConfigDict

PYDANTIC_V2 = int(VERSION.split(".", 1)[0]) == 2

T = TypeVar("T")

if PYDANTIC_V2:  # pragma: pydantic-v2
    from pydantic import (
        TypeAdapter,
        field_validator as field_validator,
        model_validator as model_validator,
    )

    # region from nonebot2

    def model_config(model: Type[BaseModel]) -> ConfigDict:
        """Get config of a model."""
        return model.model_config

    def model_dump(
        model: BaseModel,
        include: Optional[Set[str]] = None,
        exclude: Optional[Set[str]] = None,
        by_alias: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> Dict[str, Any]:
        return model.model_dump(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )

    def type_validate_python(type_: Type[T], data: Any) -> T:
        """Validate data with given type."""
        return TypeAdapter(type_).validate_python(data)

    def type_validate_json(type_: Type[T], data: Union[str, bytes]) -> T:
        """Validate JSON with given type."""
        return TypeAdapter(type_).validate_json(data)

    # endregion

    def get_model_with_config(config: ConfigDict) -> Type[BaseModel]:
        class Model(BaseModel):
            model_config = config

        return Model

else:  # pragma: pydantic-v1
    from pydantic import parse_obj_as, parse_raw_as, root_validator, validator

    if TYPE_CHECKING:
        from pydantic import _V2BeforeAfterOrPlainValidatorType, _V2WrapValidatorType

    # region from nonebot2

    def model_config(model: Type[BaseModel]) -> ConfigDict:
        """Get config of a model."""
        return (
            model.__config__
            if isinstance(model.__config__, dict)
            else ConfigDict(
                {
                    k: v
                    for k, v in model.__config__.__dict__.items()
                    if not (k.startswith("__") and k.endswith("__"))
                },
            )
        )

    def model_dump(
        model: BaseModel,
        include: Optional[Set[str]] = None,
        exclude: Optional[Set[str]] = None,
        by_alias: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> Dict[str, Any]:
        return model.dict(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )

    def type_validate_python(type_: Type[T], data: Any) -> T:
        """Validate data with given type."""
        return parse_obj_as(type_, data)

    def type_validate_json(type_: Type[T], data: Union[str, bytes]) -> T:
        """Validate JSON with given type."""
        return parse_raw_as(type_, data)

    # endregion

    @overload
    def model_validator(*, mode: Literal["before"]): ...

    @overload
    def model_validator(*, mode: Literal["after"]): ...

    def model_validator(*, mode: Literal["before", "after"]):
        return root_validator(
            pre=mode == "before",  # type: ignore
            allow_reuse=True,
        )

    @overload
    def field_validator(
        __field: str,  # noqa: PYI063
        *fields: str,
        mode: Literal["before", "after", "plain"] = ...,
        check_fields: Optional[bool] = ...,
    ) -> Callable[
        ["_V2BeforeAfterOrPlainValidatorType"],
        "_V2BeforeAfterOrPlainValidatorType",
    ]: ...

    @overload
    def field_validator(
        __field: str,  # noqa: PYI063
        *fields: str,
        mode: Literal["wrap"],
        check_fields: Optional[bool] = ...,
    ) -> Callable[["_V2WrapValidatorType"], "_V2WrapValidatorType"]: ...

    def field_validator(
        __field: str,  # noqa: PYI063
        *fields: str,
        mode: Literal["before", "after", "wrap", "plain"] = "after",
        check_fields: Optional[bool] = None,  # noqa: ARG001
    ):
        return validator(__field, *fields, pre=(mode == "before"), allow_reuse=True)

    def get_model_with_config(config: ConfigDict) -> Type[BaseModel]:
        class Model(BaseModel, **config):
            pass

        return Model



# generated by copilot

import json
from typing import Optional

from cookit.pyd import (
    get_model_with_config,
    model_config,
    model_dump,
    type_validate_json,
    type_validate_python,
)
from pydantic import BaseModel, ConfigDict

# region from nonebot2


async def test_model_dump():
    class TestModel(BaseModel):
        test1: int
        test2: int

    assert model_dump(TestModel(test1=1, test2=2), include={"test1"}) == {"test1": 1}
    assert model_dump(TestModel(test1=1, test2=2), exclude={"test1"}) == {"test2": 2}


async def test_validate():
    class TestModel(BaseModel):
        test1: int
        test2: str
        test3: bool
        test4: dict
        test5: list
        test6: Optional[int]

    data = {
        "test1": 1,
        "test2": "2",
        "test3": True,
        "test4": {},
        "test5": [],
        "test6": None,
    }
    expected = TestModel(test1=1, test2="2", test3=True, test4={}, test5=[], test6=None)
    assert type_validate_python(TestModel, data) == expected
    assert type_validate_json(TestModel, json.dumps(data)) == expected


# endregion


async def test_get_model_with_config():
    config = ConfigDict(arbitrary_types_allowed=True)
    Model = get_model_with_config(config)  # noqa: N806
    assert isinstance(Model(), Model)
    for k in config:
        assert model_config(Model)[k] == config[k]



def camel_case(string: str, upper_first: bool = False) -> str:
    pfx, *rest = string.split("_")
    if upper_first:
        pfx = pfx.capitalize()
    sfx = "".join(x.capitalize() for x in rest)
    return f"{pfx}{sfx}"


def full_to_half(text: str) -> str:
    return "".join(
        chr(ord(char) - 0xFEE0) if "\uff01" <= char <= "\uff5e" else char
        for char in text
    ).replace("\u3000", " ")


def escape_single_quotes(value: str) -> str:
    return value.replace("'", "\\'")


def escape_double_quotes(value: str) -> str:
    return value.replace('"', '\\"')


def escape_backticks(value: str) -> str:
    return value.replace("`", "\\`")



import pytest

from .utils import LoggingContext


def test_logged_suppress():
    from cookit.loguru import logged_suppress

    def value_error():
        raise ValueError("val")

    def index_error():
        raise IndexError

    with LoggingContext() as ctx:
        with logged_suppress("test1"):
            value_error()
        ctx.should_log(message="test1", level_str="ERROR", exception=ValueError)

        with logged_suppress("test2", ValueError):
            value_error()
        ctx.should_log(message="test2", level_str="ERROR", exception=ValueError)

        try:
            with logged_suppress("test3", ValueError):
                index_error()
        except IndexError:
            pass
        else:
            pytest.fail("IndexError should be raised")

        with logged_suppress(
            lambda e: f"test4: {e.args[0]}",
            ValueError,
            IndexError,
            level="WARNING",
        ):
            value_error()
        ctx.should_log(
            message="test4: val",
            level_str="WARNING",
            exception=ValueError,
        )

        with logged_suppress(
            lambda e: f"test5: {type(e).__name__}",
            ValueError,
            IndexError,
            level="DEBUG",
        ):
            index_error()
        ctx.should_log(
            message="test5: IndexError",
            level_str="DEBUG",
            exception=IndexError,
        )

        with logged_suppress("test6", log_stack=False, debug_stack=True):
            value_error()
        ctx.should_log(message="test6", level_str="ERROR", exception=None)
        ctx.should_log(message="Stacktrace", level_str="DEBUG", exception=ValueError)



import asyncio as aio
import random
from typing import List, Optional, Tuple, Union

from nonebot import logger
from nonebot.adapters import Bot as BaseBot, Event as BaseEvent
from nonebot_plugin_alconna.uniseg import (  # noqa: E402
    Receipt,
    Reply,
    Target,
    UniMessage,
)


class RecallContext:
    def __init__(
        self,
        delay: Union[float, Tuple[float, float], None] = None,
        wait: bool = False,
    ) -> None:
        self.delay = delay
        self.wait = wait
        self.receipts: List[Receipt] = []

    async def __aenter__(self) -> "RecallContext":
        return self

    async def __aexit__(self, *_) -> None:
        task = aio.create_task(self.recall())
        if self.wait:
            await task

    def _get_delay(self) -> float:
        if isinstance(self.delay, tuple):
            return random.uniform(*self.delay)
        return self.delay or 0

    def append(self, r: Receipt):
        self.receipts.append(r)

    async def send(
        self,
        message: Union[UniMessage, str],
        target: Union[BaseEvent, Target, None] = None,
        bot: Optional[BaseBot] = None,
        fallback: bool = True,
        at_sender: Union[str, bool] = False,
        reply_to: Union[str, bool, Reply, None] = False,
    ) -> None:
        self.append(
            await (
                UniMessage.text(message) if isinstance(message, str) else message
            ).send(
                target=target,
                bot=bot,
                fallback=fallback,
                at_sender=at_sender,
                reply_to=reply_to,
            ),
        )

    @staticmethod
    async def safe_recall(r: Receipt) -> None:
        try:
            await r.recall()
        except Exception as e:
            logger.warning(f"Recall failed: {type(e).__name__}: {e}")
            logger.opt(exception=e).debug("Stack trace:")

    async def recall(self) -> None:
        if not self.receipts:
            return

        receipts = self.receipts.copy()
        self.receipts.clear()

        if self.delay is None:
            await aio.gather(*(self.safe_recall(r) for r in receipts))
        else:
            for r in receipts:
                await aio.sleep(self._get_delay())
                await self.safe_recall(r)



from typing import Callable, Type
from typing_extensions import TypeAlias

from pydantic import BaseModel, ConfigDict

from .. import camel_case
from .compat import get_model_with_config

AliasFuncType: TypeAlias = Callable[[str], str]


def get_alias_model(alias_func: AliasFuncType) -> Type[BaseModel]:
    return get_model_with_config(ConfigDict(alias_generator=alias_func))


CamelAliasModel = get_alias_model(camel_case)



"""Install `cookit[jinja]` before import this module."""

from . import filters as filters
from .common import (
    make_register_jinja_filter_deco as make_register_jinja_filter_deco,
)
from .filters import (
    all_filters as all_filters,
    register_all_filters as register_all_filters,
)



import asyncio as aio

import pytest


async def test_with_semaphore():
    from cookit import with_semaphore

    ret = "Hello, World!"
    semaphore = aio.Semaphore(1)

    @with_semaphore(semaphore)
    async def async_func():
        return ret

    result = await aio.gather(
        async_func(),
        async_func(),
    )
    assert tuple(result) == (ret, ret)


async def test_queued():
    from cookit import queued

    ret = "Hello, World!"
    counter = 0

    @queued
    async def async_func():
        nonlocal counter
        await aio.sleep(0)
        counter += 1
        return f"{ret}{counter}"

    result = await aio.gather(
        async_func(),
        async_func(),
        async_func(),
        async_func(),
        async_func(),
    )
    assert tuple(result) == (f"{ret}1", f"{ret}2", f"{ret}3", f"{ret}4", f"{ret}5")


async def test_race():
    from cookit import race

    async def f1():
        await aio.sleep(1)
        return 1

    async def f2():
        await aio.sleep(2)
        return 2

    async def fe():
        raise Exception

    assert await race(f1(), f2()) == 1
    with pytest.raises(Exception):  # noqa: B017, PT011
        await race(f1(), f2(), fe())



from .base import (
    LazyGetterType as LazyGetterType,
    append_func_to_dict_deco as append_func_to_dict_deco,
    auto_delete as auto_delete,
    chunks as chunks,
    flatten as flatten,
    lazy_get as lazy_get,
    make_append_func_to_dict_deco as make_append_func_to_dict_deco,
    qor as qor,
    set_default as set_default,
    to_b64_url as to_b64_url,
)
from .pagination import (
    IterPFKwargs as IterPFKwargs,
    PaginationCallable as PaginationCallable,
    iter_pagination_func as iter_pagination_func,
)



# generated by copilot

from typing import List


def get_mock_pagination_func(max_size: int):
    async def mock_pagination_func(page_size: int, offset: int) -> List[int]:
        return list(range(offset, min(offset + page_size, max_size)))

    return mock_pagination_func


async def test_iter_pagination_func():
    from cookit import iter_pagination_func

    func = iter_pagination_func(page_size=3, offset=0)(get_mock_pagination_func(9))
    lst = [x async for x in func()]
    assert lst == [0, 1, 2, 3, 4, 5, 6, 7, 8]


async def test_iter_pagination_func_with_offset():
    from cookit import iter_pagination_func

    func = iter_pagination_func(page_size=3, offset=2)(get_mock_pagination_func(9))
    lst = [x async for x in func()]
    assert lst == [2, 3, 4, 5, 6, 7, 8]


async def test_iter_pagination_func_with_max_size():
    from cookit import iter_pagination_func

    func = iter_pagination_func(page_size=3, offset=0, max_size=5)(
        get_mock_pagination_func(9),
    )
    lst = [x async for x in func()]
    assert lst == [0, 1, 2, 3, 4]



from datetime import timedelta


def test_auto_convert_unit():
    from cookit import auto_convert_unit

    assert auto_convert_unit(["m", "km"], multiplier=1000, value=5000) == "5.00 km"
    assert (
        auto_convert_unit(
            ["m", "km"],
            multiplier=1000,
            value=5000,
            round_n=0,
            with_space=False,
        )
        == "5km"
    )
    assert (
        auto_convert_unit(
            ["s", "min(s)", "hr(s)"],
            multiplier=60,
            value=3600,
            round_n=1,
            suffix="/it",
            with_space=False,
            unit_index=1,
        )
        == "1.0hr(s)/it"
    )
    assert (
        auto_convert_unit(
            ["b", "k", "m", "g"],
            multiplier=1024,
            value=1000,
            round_n=2,
            with_space=False,
            unit_threshold=1000,
        )
        == "0.98k"
    )
    assert (
        auto_convert_unit(
            ["b", "k", "m", "g"],
            multiplier=1024,
            value=1024,
            round_n=1,
            suffix="/s",
            with_space=True,
            unit_threshold=1000,
        )
        == "1.0 k/s"
    )


td_full = timedelta(days=1, hours=2, minutes=3, seconds=4, milliseconds=5)
td_no_day = timedelta(hours=2, minutes=3, seconds=4, milliseconds=5)
td_zero = timedelta()


def test_format_timedelta():
    from cookit import (
        TimeUnit,
        format_timedelta,
        format_timedelta_human_en,
        format_timedelta_human_en_spc,
        format_timedelta_human_zh,
        format_timedelta_human_zh_spc,
    )

    # normal
    assert format_timedelta(td_full) == "1:02:03:04"
    assert format_timedelta(td_full, with_ms=True) == "1:02:03:04.005"
    assert format_timedelta(td_full, with_ms=True, ms_num=1) == "1:02:03:04.0"
    assert format_timedelta(td_full, with_ms=True, ms_num=None) == "1:02:03:04.5"
    assert format_timedelta(td_full, pad_day=True) == "01:02:03:04"
    assert format_timedelta(td_full, pad_day=True, day_pad_num=3) == "001:02:03:04"
    assert format_timedelta(td_full, pad_time=False) == "1:2:3:4"
    assert format_timedelta(td_full, pad_time_first=False) == "1:2:03:04"
    assert format_timedelta(td_full, pad_time=False, pad_time_first=False) == "1:2:3:4"
    assert (
        format_timedelta(td_full, unit_up_to=TimeUnit.HOUR, with_ms=True)
        == "26:03:04.005"
    )
    assert (
        format_timedelta(td_full, unit_up_to=TimeUnit.MINUTE, with_ms=True)
        == "1563:04.005"
    )
    assert (
        format_timedelta(td_full, unit_up_to=TimeUnit.SECOND, with_ms=True)
        == "93784.005"
    )
    assert (
        format_timedelta(td_full, unit_up_to=TimeUnit.MILLISECOND, with_ms=True)
        == "93784005"
    )

    # no day
    assert format_timedelta(td_no_day) == "02:03:04"
    assert format_timedelta(td_no_day, with_ms=True) == "02:03:04.005"
    assert format_timedelta(td_no_day, with_ms=True) == "02:03:04.005"
    assert format_timedelta(td_no_day, pad_time_first=False) == "2:03:04"
    assert (
        format_timedelta(td_no_day, with_ms=True, with_empty_day=True)
        == "0:02:03:04.005"
    )

    # zero
    assert (
        format_timedelta(td_zero, with_ms=True, with_empty_day=True) == "0:00:00:00.000"
    )

    # human
    assert format_timedelta_human_en(td_full, with_ms=True) == "1d2h3m4s5ms"
    assert format_timedelta_human_en_spc(td_full, with_ms=False) == "1d 2h 3m 4s"
    assert format_timedelta_human_zh(td_full, with_ms=False) == "1天2时3分4秒"
    assert (
        format_timedelta_human_zh_spc(td_full, with_ms=True) == "1天 2时 3分 4秒 5毫秒"
    )



# generated by copilot


from typing import Callable, Dict, cast


def test_lazy_get():
    from cookit import LazyGetterType, lazy_get

    a = cast("LazyGetterType[int, []]", 5)
    result = lazy_get(a)
    assert result == a

    b = lambda: 10  # noqa: E731
    result = lazy_get(b)
    assert result == b()

    c = cast(Callable[[int], int], lambda x: x + 1)  # noqa: E731
    result = lazy_get(c, 1)
    assert result == c(1)


def test_qor_with_a_not_none():
    from cookit import qor

    a = 5
    b = 10
    result = qor(a, b)
    assert result == a


def test_qor_with_a_none_and_b_not_callable():
    from cookit import qor

    a = None
    b = 10
    result = qor(a, b)
    assert result == b


def test_qor_with_a_none_and_b_callable():
    from cookit import qor

    a = None
    b = lambda: 10  # noqa: E731
    result = qor(a, b)
    assert result == b()


def test_chunks():
    from cookit import chunks

    lst = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    assert list(chunks(lst, 3)) == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
    lst2 = [1, 2, 3, 4, 5, 6, 7, 8]
    assert list(chunks(lst2, 3)) == [[1, 2, 3], [4, 5, 6], [7, 8]]


def test_flatten():
    from cookit import flatten

    lst = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
    assert flatten(lst) == [1, 2, 3, 4, 5, 6, 7, 8, 9]


def test_set_default_with_existing_key():
    from cookit import set_default

    target = {"a": 1, "b": 2}
    key = "a"
    default = 10
    result = set_default(target, key, default)
    assert result == target[key]
    assert len(target) == 2


def test_set_default_with_new_key():
    from cookit import set_default

    target = {"a": 1, "b": 2}
    key = "c"
    default = 10
    result = set_default(target, key, default)
    assert result == default
    assert target[key] == default
    assert len(target) == 3


def test_set_default_with_callable_default():
    from cookit import set_default

    target = {"a": 1, "b": 2}
    key = "c"
    default = lambda: 10  # noqa: E731
    result = set_default(target, key, default)
    assert result == default()
    assert target[key] == default()
    assert len(target) == 3


def test_auto_delete():
    from cookit import auto_delete

    # Test case 1: No transformation function provided
    target1 = {"a": 1, "b": 2, "c": None, "d": 0}
    expected_result1 = {"a": 1, "b": 2}
    result1 = auto_delete(target1)
    assert result1 == expected_result1
    assert target1 == expected_result1

    # Test case 2: Transformation function provided
    def transform_func(value: int):
        return value * 2 if value % 2 == 0 else None

    target2 = {"a": 1, "b": 2, "c": 3, "d": 4}
    expected_result2 = {"b": 2, "d": 4}
    expected_result2_transformed = {"b": 4, "d": 8}
    result2 = auto_delete(target2, transform=transform_func)
    assert result2 == expected_result2_transformed
    assert target2 == expected_result2

    # Test case 3: Empty target dictionary
    target3 = {}
    expected_result3 = {}
    result3 = auto_delete(target3)
    assert result3 == expected_result3
    assert target3 == expected_result3


def test_append_func_to_dict_deco():
    from cookit import append_func_to_dict_deco, make_append_func_to_dict_deco

    def func1():
        return 1

    def func2():
        return 2

    func_dict1: Dict[str, Callable] = {}
    append_func_to_dict_deco(func_dict1, func1)
    append_func_to_dict_deco(func_dict1, "func_func2")(func2)
    assert func_dict1 == {"func1": func1, "func_func2": func2}

    func_dict2: Dict[str, Callable] = {}
    append_func = make_append_func_to_dict_deco(func_dict2)
    append_func(func1)
    append_func("func_func2")(func2)
    assert func_dict2 == {"func1": func1, "func_func2": func2}



from typing import (
    Generic,
    Iterable,
    List,
    Optional,
    SupportsIndex,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Self, deprecated

T = TypeVar("T")
TB = TypeVar("TB")


@deprecated("Use `collections.deque` instead")
class SizedList(List[T], Generic[T]):
    def __init__(
        self,
        iterable: Optional[Iterable[T]] = None,
        size: Optional[int] = None,
    ) -> None:
        if iterable is None:
            super().__init__()
        else:
            super().__init__(iterable)
        self.size = size
        self._handle_overflow()

    @property
    def last(self) -> Optional[T]:
        if self:
            return self[-1]
        return None

    def _handle_overflow(self) -> None:
        if self.size is None:
            return
        while len(self) > self.size:
            self.pop(0)

    def append(self, item: T) -> None:
        super().append(item)
        self._handle_overflow()

    def extend(self, items: Iterable[T]) -> None:
        super().extend(items)
        self._handle_overflow()

    def insert(self, index: SupportsIndex, item: T) -> None:
        super().insert(index, item)
        self._handle_overflow()

    @overload
    def __add__(self, items: Iterable[T]) -> "SizedList[T]": ...

    @overload
    def __add__(self, items: Iterable[TB]) -> "SizedList[Union[T, TB]]": ...

    def __add__(self, items):
        return SizedList((*self, *items), size=self.size)

    def __iadd__(self, items: Iterable[T]) -> Self:
        self.extend(items)
        return self



from datetime import timedelta
from enum import IntEnum
from functools import partial
from typing import Any, Optional, Sequence


def auto_convert_unit(
    units: Sequence[str],
    multiplier: int,
    value: float,
    round_n: int = 2,
    suffix: str = "",
    with_space: bool = True,
    unit_index: int = 0,
    unit_threshold: Optional[int] = None,
) -> str:
    if unit_threshold is None:
        unit_threshold = multiplier
    units = units[unit_index:]
    if not units:
        raise ValueError("Wrong `unit_index`")

    unit = None
    for x in units:
        if value < unit_threshold:
            unit = x
            break
        value /= multiplier
    return (
        f"{value:.{round_n}f}"
        f"{' ' if with_space else ''}"
        f"{unit or units[-1]}"
        f"{suffix}"
    )


BYTE_UNITS = ["B", "K", "M", "G", "T", "P"]
BYTE_B_UNITS = ["B", "KB", "MB", "GB", "TB", "PB"]

auto_convert_byte = partial(
    auto_convert_unit,
    units=BYTE_UNITS,
    multiplier=1024,
    unit_threshold=1000,
)
auto_convert_byte_speed = partial(
    auto_convert_unit,
    units=BYTE_UNITS,
    multiplier=1024,
    suffix="/s",
    unit_threshold=1000,
)
auto_convert_byte_b = partial(
    auto_convert_unit,
    units=BYTE_B_UNITS,
    multiplier=1024,
    unit_threshold=1000,
)
auto_convert_byte_b_speed = partial(
    auto_convert_unit,
    units=BYTE_B_UNITS,
    multiplier=1024,
    suffix="/s",
    unit_threshold=1000,
)


class TimeUnit(IntEnum):
    MILLISECOND = 0
    SECOND = 1
    MINUTE = 2
    HOUR = 3
    DAY = 4


def format_timedelta(
    t: timedelta,
    unit_up_to: TimeUnit = TimeUnit.DAY,
    with_ms: bool = False,
    with_empty_day: bool = False,
    day_divider: str = ":",
    time_divider: str = ":",
    ms_divider: str = ".",
    day_suffix: str = "",
    hour_suffix: str = "",
    minute_suffix: str = "",
    second_suffix: str = "",
    ms_suffix: str = "",
    pad_day: bool = False,
    pad_time: bool = True,
    pad_time_first: bool = True,
    day_pad_num: int = 2,
    ms_num: Optional[int] = 3,
) -> str:
    mm, ss = divmod(t.seconds, 60)
    hh, mm = divmod(mm, 60)
    ms = t.microseconds // 1000

    d_str = ""
    h_str = ""
    m_str = ""
    s_str = ""
    ms_str = ""

    should_pad_next_time = pad_time_first

    def pad_t(x: Any) -> str:
        nonlocal should_pad_next_time
        if not pad_time:
            return f"{x}"
        if not should_pad_next_time:
            should_pad_next_time = True
            return f"{x}"
        return f"{x:02d}"

    if t.days or with_empty_day:
        if unit_up_to >= TimeUnit.DAY:
            dd = f"{t.days:0{day_pad_num}d}" if pad_day else f"{t.days}"
            d_str = f"{dd}{day_suffix}{day_divider}"
        else:
            hh += t.days * 24

    if unit_up_to >= TimeUnit.HOUR:
        h_str = f"{pad_t(hh)}{hour_suffix}{time_divider}"
    elif hh:
        mm += hh * 60

    if unit_up_to >= TimeUnit.MINUTE:
        m_str = f"{pad_t(mm)}{minute_suffix}{time_divider}"
    elif mm:
        ss += mm * 60

    if unit_up_to >= TimeUnit.SECOND:
        s_str = f"{pad_t(ss)}{second_suffix}"
    elif ss:
        ms += ss * 1000

    if unit_up_to is TimeUnit.MILLISECOND:
        with_ms = True
        ms_divider = ""

    if with_ms and ms_num != 0:
        ms = ms if ((ms_num is None) or (ms >= 1000)) else f"{ms:03d}"[:ms_num]
        ms_str = f"{ms_divider}{ms}{ms_suffix}"

    return f"{d_str}{h_str}{m_str}{s_str}{ms_str}"


TIMEDELTA_NO_PAD = {
    "pad_day": False,
    "pad_time": False,
    "ms_num": None,
}
TIMEDELTA_EMPTY_DIVIDER = {
    "day_divider": "",
    "time_divider": "",
    "ms_divider": "",
}
TIMEDELTA_SPACE_DIVIDER = {
    "day_divider": " ",
    "time_divider": " ",
    "ms_divider": " ",
}
TIMEDELTA_SUFFIX_EN = {
    "day_suffix": "d",
    "hour_suffix": "h",
    "minute_suffix": "m",
    "second_suffix": "s",
    "ms_suffix": "ms",
}
TIMEDELTA_SUFFIX_ZH = {
    "day_suffix": "天",
    "hour_suffix": "时",
    "minute_suffix": "分",
    "second_suffix": "秒",
    "ms_suffix": "毫秒",
}

format_timedelta_human_en = partial(
    format_timedelta,
    **TIMEDELTA_NO_PAD,
    **TIMEDELTA_EMPTY_DIVIDER,
    **TIMEDELTA_SUFFIX_EN,
)
format_timedelta_human_en_spc = partial(
    format_timedelta,
    **TIMEDELTA_NO_PAD,
    **TIMEDELTA_SPACE_DIVIDER,
    **TIMEDELTA_SUFFIX_EN,
)
format_timedelta_human_zh = partial(
    format_timedelta,
    **TIMEDELTA_NO_PAD,
    **TIMEDELTA_EMPTY_DIVIDER,
    **TIMEDELTA_SUFFIX_ZH,
)
format_timedelta_human_zh_spc = partial(
    format_timedelta,
    **TIMEDELTA_NO_PAD,
    **TIMEDELTA_SPACE_DIVIDER,
    **TIMEDELTA_SUFFIX_ZH,
)
